@@ -23,11 +23,11 @@
 #include <config.h>
 #include <gnome.h>
 #include <string.h>
-#include <dirent.h>
 #include <gconf/gconf-client.h>
 #include <games-gconf.h>
 #include <games-clock.h>
 #include <games-frame.h>
+#include <games-files.h>
 
 #include "properties.h"
 #include "gnothello.h"
@@ -42,7 +42,6 @@
 #define KEY_SHOW_GRID "/apps/iagno/show_grid"
 #define KEY_FLIP_FINAL_RESULTS "/apps/iagno/flip_final_results"
 
-static GtkWidget *propbox = NULL;
 
 extern GtkWidget *window;
 extern GtkWidget *time_display;
@@ -69,7 +68,7 @@ gint t_animate_stagger;
 gint t_flip_final;
 gint t_grid;
 
-GList * theme_list = NULL;
+static GamesFileList * theme_file_list = NULL;
 
 static void apply_changes (void);
 
@@ -417,35 +416,26 @@ apply_changes (void)
 	save_properties ();
 }
 
-static void
-close_cb (GtkWidget *widget, gint arg1, gpointer data)
+static gboolean
+close_cb (GtkWidget *widget)
 {
 	gtk_widget_hide (widget);
 
-/*	if (arg1 == GTK_RESPONSE_REJECT)
-		return;
-
-	apply_changes ();
-	
-	save_properties (); */
-}
-
-static void
-destroy_cb (GtkWidget *widget, gpointer data)
-{
-
+	return TRUE;
 }
 
 void
 set_selection (GtkWidget *widget, gpointer data)
 {
-	GList * entry;
+	gchar * filename;
 
-	entry = g_list_nth (theme_list,
-			    gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
-	
-	g_free (tile_set_tmp);
-	tile_set_tmp = g_strdup (entry->data);
+	if (tile_set_tmp)
+		g_free (tile_set_tmp);
+
+	filename = games_file_list_get_nth (theme_file_list,
+					    gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
+		
+	tile_set_tmp = g_strdup (filename);
 
 	apply_changes ();
 }
@@ -456,58 +446,29 @@ free_str (GtkWidget *widget, void *data)
         g_free(data);
 }
 
-void
-fill_menu (GtkWidget *menu)
+static GtkWidget *
+fill_menu (void)
 {
-        struct dirent *e;
-        char *dname = NULL;
-        DIR *dir;
-        int itemno = 0;
-	gboolean found_default = FALSE;
+        gchar *dname = NULL;
+
+	/* FIXME: we need to check that both dname is valid and that
+	 * games_file_list_new_images returns something. */
 
 	dname = gnome_program_locate_file (NULL,
-			GNOME_FILE_DOMAIN_APP_PIXMAP,  "iagno", FALSE, NULL);
-        dir = opendir(dname);
+					   GNOME_FILE_DOMAIN_APP_PIXMAP, 
+					   "iagno", FALSE, NULL);
 
-	if (theme_list) {
-		g_list_foreach (theme_list, (GFunc) g_free, NULL);
-		g_list_free (theme_list);
-		theme_list = NULL;
-	}
-	
-        if(!dir)
-                return;
+	if (theme_file_list)
+		g_object_unref (theme_file_list);
 
-        while((e = readdir(dir)) != NULL) {
-                char *s = g_strdup(e->d_name);
-                if(strstr(e->d_name, ".png") == 0) {
-                        g_free(s);
-                        continue;
-                }
-
-		theme_list = g_list_append (theme_list, s);
-		gtk_combo_box_append_text (GTK_COMBO_BOX (menu), s);
-
-		if (strcmp(tile_set, s) == 0) {
-			found_default = TRUE;
-			gtk_combo_box_set_active(GTK_COMBO_BOX(menu), itemno);
-		}
-
-                itemno++;
-        }
-        closedir(dir);
+	theme_file_list = games_file_list_new_images (dname, NULL);
 	g_free (dname);
 
-	/* FIXME: Should popup an error dialog if no themes were found */
-	if (itemno == 0) {
-		gtk_widget_set_sensitive (menu, FALSE);
-		return;
-	} 
+	games_file_list_transform_basename (theme_file_list);
 
-	if (! found_default) {
-		/* Set the theme arbitrarily to be the first one. */
-		gtk_combo_box_set_active(GTK_COMBO_BOX(menu), 0);
-	}
+	return games_file_list_create_widget (theme_file_list, tile_set,
+					      GAMES_FILE_LIST_REMOVE_EXTENSION |
+					      GAMES_FILE_LIST_REPLACE_UNDERSCORES);
 }
 
 void
@@ -521,21 +482,20 @@ show_properties_dialog (void)
 	GtkWidget *button;
 	GtkWidget *frame;
 	GtkWidget *vbox, *vbox2;
+	GtkWidget *propbox = NULL;
 	GtkWidget *option_menu;
 
-	if (propbox)
-	{
+	if (propbox) {
                 gtk_window_present (GTK_WINDOW (propbox));
                 return;
 	}
 
 	reset_properties ();
-	
-	propbox = gtk_dialog_new_with_buttons (NULL,
+
+	propbox = gtk_dialog_new_with_buttons (_("Iagno Preferences"),
 			GTK_WINDOW (window),
 			0,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
-        gtk_window_set_title (GTK_WINDOW(propbox), _("Iagno Preferences"));
         
         gtk_dialog_set_has_separator (GTK_DIALOG (propbox), FALSE);
 	notebook = gtk_notebook_new ();
@@ -559,7 +519,7 @@ show_properties_dialog (void)
 	button = gtk_check_button_new_with_label (_("Use quick moves"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
                                       (computer_speed == COMPUTER_MOVE_DELAY / 2));
-	g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+	g_signal_connect (G_OBJECT (button), "toggled", G_CALLBACK
                           (quick_moves_select), NULL);
 	
 	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
@@ -575,8 +535,8 @@ show_properties_dialog (void)
 	if (black_computer_level == 0)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (black_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (black_computer_level_select),
 			(gpointer) 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -585,8 +545,8 @@ show_properties_dialog (void)
 	if (black_computer_level == 1)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (black_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (black_computer_level_select),
 			(gpointer) 1);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -595,8 +555,8 @@ show_properties_dialog (void)
 	if (black_computer_level == 2)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (black_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (black_computer_level_select),
 			(gpointer) 2);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -605,8 +565,8 @@ show_properties_dialog (void)
 	if (black_computer_level == 3)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (black_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (black_computer_level_select),
 			(gpointer) 3);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -622,8 +582,8 @@ show_properties_dialog (void)
 	if (white_computer_level == 0)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (white_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (white_computer_level_select),
 			(gpointer) 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -632,8 +592,8 @@ show_properties_dialog (void)
 	if (white_computer_level == 1)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-                          GTK_SIGNAL_FUNC (white_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+                          G_CALLBACK (white_computer_level_select),
                           (gpointer) 1);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -642,8 +602,8 @@ show_properties_dialog (void)
 	if (white_computer_level == 2)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-                          GTK_SIGNAL_FUNC (white_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+                          G_CALLBACK (white_computer_level_select),
                           (gpointer) 2);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -653,8 +613,8 @@ show_properties_dialog (void)
 	if (white_computer_level == 3)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (white_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (white_computer_level_select),
 			(gpointer) 3);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -675,8 +635,8 @@ show_properties_dialog (void)
 	if (animate == 0)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (animate_select), (gpointer) 0);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (animate_select), (gpointer) 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -684,8 +644,8 @@ show_properties_dialog (void)
 	if (animate == 1)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (animate_select), (gpointer) 1);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (animate_select), (gpointer) 1);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -693,8 +653,8 @@ show_properties_dialog (void)
 	if (animate == 2)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (animate_select), (gpointer) 2);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (animate_select), (gpointer) 2);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	
@@ -707,24 +667,24 @@ show_properties_dialog (void)
 	button = gtk_check_button_new_with_label (_("Stagger flips"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 			t_animate_stagger);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (animate_stagger_select), NULL);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (animate_stagger_select), NULL);
 	
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	button = gtk_check_button_new_with_label (_("Show grid"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 			t_grid);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (grid_select), NULL);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (grid_select), NULL);
 	
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	button = gtk_check_button_new_with_label (_("Flip final results"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 			t_flip_final);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (flip_final_select), NULL);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (flip_final_select), NULL);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	
@@ -734,10 +694,9 @@ show_properties_dialog (void)
 	
 	gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
 	
-	option_menu = gtk_combo_box_new_text ();
-	fill_menu (option_menu);
-	g_signal_connect(GTK_OBJECT(option_menu), "changed",
-			 (GtkSignalFunc)set_selection, NULL);
+	option_menu = fill_menu ();
+	g_signal_connect(G_OBJECT(option_menu), "changed",
+			 G_CALLBACK (set_selection), NULL);
 	gtk_box_pack_start (GTK_BOX (hbox), option_menu, TRUE, TRUE, 0);
 	
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -747,12 +706,10 @@ show_properties_dialog (void)
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table,
                                   label);
         
-	g_signal_connect (GTK_OBJECT (propbox), "response", GTK_SIGNAL_FUNC
+	g_signal_connect (G_OBJECT (propbox), "response", G_CALLBACK
+			(close_cb), NULL);
+	g_signal_connect (G_OBJECT (propbox), "delete_event", G_CALLBACK
 			(close_cb), NULL);
-	g_signal_connect (GTK_OBJECT (propbox), "destroy", GTK_SIGNAL_FUNC
-			(destroy_cb), NULL);
-	g_signal_connect (GTK_OBJECT (propbox), "close", GTK_SIGNAL_FUNC
-			(destroy_cb), NULL);
 
 	gtk_widget_show_all (propbox);
 }
